library(MASS)
library(mvtnorm) # multivariate normal random numbers 

################################################################################

ptm <- proc.time()

#==============================================================================#
# Table with different settings based on combinations of different values for n, \sigma^2_a,\sigma^2_b and \rho_{ab}

tabela <- matrix(0,80,12)
aux <- matrix(c(4,2,0.75,1, 4,2,0.25,1, 2,2,0.75,1, 2,2,0.25,1, 2,1,0.75,1,2,1,
                0.25,1, 1,2,0.75,1,1,2,0.25,1, 1,1,0.75,1,1,1,0.25,1,1,0.5,0.75,
                1,1,0.5,0.25,1,0.5,1,0.75,1,0.5,1,0.25,1,0.5,0.5,0.75,1,0.5,0.5,
                0.25,1),16,4,byrow = T)

tabela[,1:4] <- kronecker(c(1,1,1,1,1),aux)
tabela[,5] <- c(rep(5,16),rep(10,16),rep(25,16),rep(50,16),rep(100,16))

# The tabela.cov table contains the results of the analysis of the profiles of the lines of the sample covariance matrix 
tabela.cov <- matrix(0,80,12)
tabela.cov[,1:4] <- kronecker(c(1,1,1,1,1),aux)
tabela.cov[,5] <- c(rep(5,16),rep(10,16),rep(25,16),rep(50,16),rep(100,16))

#==============================================================================#
# beginning of the simulation
for (itera in 1:dim(tabela)[1]) {

  m <- tabela[itera,5] # Number of units
  np <- 3 # Number of parameters
  nobs <- 5  # Number of observations
  peso <- sqrt((m/(m - 1))) # weights as shown in equation (8) 
  peso.cov <- sqrt((nobs/(nobs - 1))) # weights as shown in equation (8) 
  nsimu <- 1000 # Number of simulations every settings
  g <- m*nobs - m*np # Degree of freedom
  f <- nobs - np
  g.cov <- nobs ^ 2 - nobs*np # Degree of freedom: line profile analysis of the sample covariance matrix
  sigma2.erro <- tabela[itera,4]
  resf <-   matrix(0,nsimu,6) # auxiliary matrix
  resf.cov <-   matrix(0,nsimu,6) # auxiliary matrix
  propmodc  <- 0 # Hit ratio
  propmodc.cov <- 0 # Hit rate analyzing the lines of the sample covariance matrix
  
#==============================================================================#
# Matrix X_j   
  x  <-  1:5
  X1j  <-  x - mean(x)
  X2j  <-  X1j ^ 2
  Xj  <-  matrix(c(rep(1,5),X1j,X2j),5,3)
#==============================================================================#
  vjk  <-   diag(solve(t(Xj) %*% Xj))
# Matrix G
  r <- matrix(0,m,14)
  r.cov <-  matrix(0,nobs,14)
  int <- rep(1,nobs)
  z <- matrix(c(int,c(1:nobs)),nobs,2)    #solve(t(z)%*%z)
  v1 <- tabela[itera,1]
  v2 <- tabela[itera,2]
  ro <- tabela[itera,3] # 0.3   
  c12 <- ro*sqrt(v1*v2)
  G <-  matrix(c(c(v1,(c12)),c((c12),v2)),2,2,byrow = T)
  #G;det(G)
# Matrix Var(y_i)
  v <- z %*% G %*% t(z) + sigma2.erro*diag(nobs)
#==============================================================================#
# Simulando amostras

  for (simu in 1:nsimu) {


     d <-  rmvnorm(m, mean = c(45,50,55,60,65),v)   #mvrnorm
     linc <- range(d)
     cov(d)
     matriz.cov <-  cov(d)
     lincov <- range(matriz.cov)
     med <- apply(d, 2, mean)
       

# Profiles plots
     plot(d[1,],ylim = c(0,linc[2] + 1),type = "l",ylab = "",xlab = "Time")
     for (i in 2:m) {
     lines(d[i,])
     }
  
     for (j in 1:m) {
        
              fit <-  lm(d[j,] ~ X1j + X2j)
              ajuste <- summary(fit)
              r[j,1:3] <- fit$coef
              r[j,4] <- ajuste$sigma ^ 2
              r[j,5] <-  (ajuste$sigma ^ 2)*(f/g)
              r[j,6] <-  fit$coef[1]/(sqrt(vjk[1]))
              r[j,7] <-  fit$coef[2]/(sqrt(vjk[2]))
              r[j,8] <-  fit$coef[3]/(sqrt(vjk[3]))
     }
     sigma2 <- sum(r[,5])
     for (j in 1:m) {
              r[j,9] <- (r[j,6] - mean(r[,6]))*(peso*sqrt((1/sigma2)))
              r[j,10] <- (r[j,7] - mean(r[,7]))*(peso*sqrt((1/sigma2)))
              r[j,11] <- (r[j,8] - mean(r[,8]))*(peso*sqrt((1/sigma2)))
              if ((r[j,9] <= -qt((1 - (0.01/(2*m*np))), g)) || 
                 (r[j,9] >= qt((1 - (0.01/(2*m*np))), g))) {
                            r[j,12] <- 1 }
              else{ 
                      r[j,12] <-  0  }
                            
              r[j,13] <- ifelse((r[j,10] <= -qt((1 - (0.01/(2*m*np))), g)) ||
                                  (r[j,10] >= qt((1 - (0.01/(2*m*np))), g)),1,0)
              r[j,14] <- ifelse((r[j,11] <= -qt((1 - (0.01/(2*m*np))), g)) ||
                                  (r[j,11] >= qt((1 - (0.01/(2*m*np))), g)),1,0)
        }  
        var.tb1 <- (1/(m ^ 2))*sigma2*m*vjk[1]
        tb1 <- mean(r[,1])/sqrt(var.tb1)
        var.tb2 <-  (1/(m ^ 2))*sigma2*m*vjk[2]
        tb2 <- mean(r[,2])/sqrt(var.tb2)           
        var.tb3 <-  (1/(m ^ 2))*sigma2*m*vjk[3]
        tb3 <- mean(r[,3])/sqrt(var.tb3)           
        resf[simu,1] <- ifelse((tb1 <= -qt((1 - (0.025)), g)) || 
                                 (tb1 >= qt((1 - (0.025)), g)),1,0)
        resf[simu,2] <- ifelse((tb2 <= -qt((1 - (0.025)), g)) ||
                                 (tb2 >= qt((1 - (0.025)), g)),1,0)
        resf[simu,3] <-  ifelse((tb3 <= -qt((1 - (0.025)), g)) || 
                                        (tb3 >= qt((1 - (0.025)), g)),1,0)
        resf[simu,4] <- ifelse((sum(r[,12])/m) > 0.01,1,0)
        resf[simu,5] <- ifelse((sum(r[,13])/m) > 0.01,1,0)
        resf[simu,6] <- ifelse((sum(r[,14])/m) > 0.01,1,0)
			
	      propmodc <- propmodc + ifelse(resf[simu,1] == 1 && resf[simu,2] == 1 &&
	                                   resf[simu,3] == 0 && resf[simu,4] == 1 &&
	                                   resf[simu,5] == 1 && resf[simu,6] == 0,1,0)	
  med.resf <- apply(resf, 2, mean); med.resf;((propmodc/nsimu)*100)
  tabela[itera,6:11] <- med.resf 
  tabela[itera,12] <- ((propmodc/nsimu)*100)

# End of the analysis of individual profiles

################################################################################
################################################################################

####### Analysis of the lines of the covariance matrix #########################
  
################################################################################
################################################################################




## gráficos de perfis intercepto aleatório ##
     plot(matriz.cov[1,],ylim = c(0,lincov[2] + 1),type = "l",ylab = "",
       xlab = "Time")
     for (i in 2:nobs) {
        lines(matriz.cov[i,])
     }
    
     for (j in 1:nobs) {
        fit.cov <- lm(matriz.cov[j,] ~ X1j + X2j)
        ajuste.cov <- summary(fit.cov)
        r.cov[j,1:3] <- fit.cov$coef
        r.cov[j,4] <- ajuste.cov$sigma ^ 2
        r.cov[j,5] <- (ajuste.cov$sigma ^ 2)*(f/g.cov)
        r.cov[j,6] <-  fit.cov$coef[1]/(sqrt(vjk[1]))
        r.cov[j,7] <-  fit.cov$coef[2]/(sqrt(vjk[2]))
        r.cov[j,8] <-  fit.cov$coef[3]/(sqrt(vjk[3]))
     }
     sigma2.cov <- sum(r.cov[,5])
     for (j in 1:nobs) {
        r.cov[j,9] <- (r.cov[j,6] - mean(r.cov[,6]))*
          (peso.cov*sqrt((1/sigma2.cov)))
        r.cov[j,10] <-  (r.cov[j,7] - mean(r.cov[,7]))*
          (peso.cov*sqrt((1/sigma2.cov)))
        r.cov[j,11] <- (r.cov[j,8] - mean(r.cov[,8]))*
          (peso.cov*sqrt((1/sigma2.cov)))
        if ((r.cov[j,9] <= -qt((1 - (0.01/(2*nobs*np))), g.cov)) ||
            (r.cov[j,9] >= qt((1 - (0.01/(2*nobs*np))), g.cov))) {
                            r.cov[j,12] <-  1 }
        else{ 
                            r.cov[j,12] <-  0  }
        r.cov[j,13] <- ifelse((r.cov[j,10] <= -qt((1 - (0.01/(2*nobs*np))), g.cov))
                              || (r.cov[j,10] >= qt((1 - (0.01/(2*nobs*np))), 
                                                    g.cov)),1,0)
        r.cov[j,14] <- ifelse((r.cov[j,11] <= -qt((1 - (0.01/(2*nobs*np))), g.cov))
                              || (r.cov[j,11] >= qt((1 - (0.01/(2*nobs*np))), 
                                                    g.cov)),1,0)
        }  
        var.tb1.cov <-  (1/(nobs ^ 2))*sigma2.cov*nobs*vjk[1]
        tb1.cov <- mean(r.cov[,1])/sqrt(var.tb1.cov)
        var.tb2.cov <-  (1/(nobs ^ 2))*sigma2.cov*nobs*vjk[2]
        tb2.cov <- mean(r.cov[,2])/sqrt(var.tb2.cov)           
        var.tb3.cov <-  (1/(nobs ^ 2))*sigma2.cov*nobs*vjk[3]
        tb3.cov <- mean(r.cov[,3])/sqrt(var.tb3.cov)           
        resf.cov[simu,1] <- ifelse((tb1.cov <= -qt((1 - (0.025)), g.cov)) ||
                                      (tb1.cov >= qt((1 - (0.025)), g.cov)),1,0)
        resf.cov[simu,2] <- ifelse((tb2.cov <= -qt((1 - (0.025)), g.cov)) ||
                                     (tb2.cov >= qt((1 - (0.025)), g.cov)),1,0)
        resf.cov[simu,3] <- ifelse((tb3.cov <= -qt((1 - (0.025)), g.cov)) ||
                                     (tb3.cov >= qt((1 - (0.025)), g.cov)),1,0)
        resf.cov[simu,4] <- ifelse((sum(r.cov[,12])/nobs) > 0.01,1,0)
        resf.cov[simu,5] <- ifelse((sum(r.cov[,13])/nobs) > 0.01,1,0)
        resf.cov[simu,6] <- ifelse((sum(r.cov[,14])/nobs) > 0.01,1,0)
			
              #propmodc.cov <- propmodc.cov+ ifelse(resf.cov[simu,1]==1 && resf.cov[simu,2]==1 && resf.cov[simu,3]==0 && resf.cov[simu,4]==1 && resf.cov[simu,5]==1 && resf.cov[simu,6]==0,1,0)	
        propmodc.cov <- propmodc.cov + ifelse(resf.cov[simu,4] == 1 &&
                                                resf.cov[simu,5] == 1 &&
                                                resf.cov[simu,6] == 0,1,0)
}
med.resf.cov <- apply(resf.cov, 2, mean); med.resf.cov;((propmodc.cov/nsimu)*100)
tabela.cov[itera,6:11] <- med.resf.cov 
tabela.cov[itera,12] <- ((propmodc.cov/nsimu)*100)
}
tabela
tabela.cov
proc.time()
proc.time() - ptm

