library(MASS)
library(Matrix)
library(mvtnorm)
library(fMultivar)
library(mnormt)
library(lattice)
library(lme4)
library(nlme)
library(RLRsim)
library(StatDA)
library(car)
library(gdata)
library(RLRsim)
library(tikzDevice)
library(ggplot2)
library(gplots)
library(heavy)
library(latticeExtra)
library(psych)
library(gmodels)
library(sciplot)
library(nortest)
library(nparLD)
library(Matrix)
library(segmented)
library(multcomp)
library(JM)
library(xlsx)
library(readxl)
library(grDevices)
library(extrafont)


#====================================================================#
#Leitura do conjunto de dados
setwd("~/Documents/2016/Artigos/Artigo_Julio&Rocha/artigo sweave e rstudio/ArtigoJAS/Example1AGA")

#dados <- read.table("//Users//franciscomarcelorocha//Documents//2012//Artigo_Julio&Rocha//Programas simulação//dados_PIG_sga_f.txt",header=T,na="NA")

data.ex1 <- read.table("dados_PIG_AIG_f.txt",header = T,na = "NA")
#====================================================================#

# Criando variáveis para ajuste dos perfis individuais

x <- 26:40
# Changing week variable to month
xc <- (x - 26) / 4.29  
#xc <- (x - mean(x)) / 4.29  
#xc <-  x  - mean(x)
xc2 <- xc ^ 2
xc3 <- xc ^ 3
np <- 3 # número de parâmetros

#====================================================================#
# 
# 
# 
min.aux <- c("week26","week27","week28","week29","week30","week31",
             "week32","week33","week34","week35","week36","week37",
             "week38","week39","week40")

aga.sga <- reshape(data.ex1, direction = "long", varying = 3:17,v.names = "Diameter",
                  timevar = "Weeks", time = as.factor(min.aux))
aga.sga <- aga.sga[order(aga.sga$id), ]
aga.sga$weeks <- rep(x, dim(data.ex1)[1], each = T)
aga.sga$month <- rep(xc, dim(data.ex1)[1], each = T)
aga.sga$subject <- aga.sga$indf

aga.sga <- na.exclude(aga.sga)

#====================================================================#
# Profiles Plots

graph1 <- xyplot(Diameter ~ weeks |  Group, groups = id, pch = 16,
            par.settings = standard.theme(quartz,color = F),
            scales = list(x = list(relation = 'same'), 
                          y = list(relation = 'same')),type = 'l',
            lty = 1,as.table = TRUE,
            panel = function(x, y, col, ...) {
              panel.xyplot(x, y, col = col, ...)
              panel.average(x, y, fun = mean, horizontal = F,
                            lwd = 4, col = 'black', ...)   
            },
            ylab = "Aorta diameter (mm/kg)", 
            xlab = expression(bold(paste("Mean Profile in bold"))),
            na.action = na.omit,data = aga.sga)

graph2 <- xyplot(Diameter ~ weeks |  Group, groups = id, pch = 16,
                 par.settings = standard.theme(quartz,color = F),
                 scales = list(x = list(relation = 'same'), 
                             y = list(relation = 'same')),
                 type = 'l',lty = 1,as.table = TRUE,
           panel = function(x, y, col, ...) {
             panel.xyplot(x, y, col = col, ...)
             panel.loess(x, y, span = .65, degree = 1,
                         family = "symmetric",lwd = 4)
           },
           ylab = " Aorta diameter (mm/kg)", xlab = "Weeks",
           sub = 'Loess in bold', ,na.action = na.omit,
           data = aga.sga)

#cairo_ps("testeAGASGA.ps",fallback_resolution = 1000,width = 13, height = 13)

#cairo_pdf("testeAGASGA.pdf",fallback_resolution = 1000,width = 13,height = 13)

#cairo_ps("FiguraS4.eps",fallback_resolution = 1000)

print(graph1, split = c(1,1,1,2), more = TRUE)
print(graph2, split = c(1, 2,1,2), more = FALSE)

######################################################################
# Modelling data
vecd <- groupedData(Diameter ~  weeks | id, data = aga.sga)
#======================================================================#
# Model 1: Suggested model

fit1 <- lme( Diameter ~ I(as.numeric(Group == 'AGA')) + 
               I(as.numeric(Group == 'SGA')) + 
               I(as.numeric(I(Group == 'AGA'))*month) +
               I(as.numeric(I(Group == 'SGA'))*month) +
               I(as.numeric(I(Group == 'SGA'))*(month ^ 2)) - 1,
             na.action = na.omit, 
             data = vecd,
             random = list(id = pdBlocked(list(
               pdSymm(~as.numeric(I(Group == 'AGA')) + 
                        I(as.numeric(I(Group == 'AGA'))*month) - 1),
               pdSymm(~as.numeric(I(Group == 'SGA')) + 
                        I(as.numeric(I(Group == 'SGA'))*month) + 
                        I(as.numeric(I(Group == 'SGA'))*(month ^ 2))
                      - 1)) )),
             control = list(MaxIter = 100000,niterEM = 100000,
                            msMaxIter = 100000,
                            tolerance = 0.0001,msTol = 0.0000001,
                            msMaxEval = 1000000)) 
summary(fit1)
round(summary(fit1)$tTable,digits = 2)
getVarCov(fit1,'random effects')
round(diag(getVarCov(fit1,'random effects')),digits = 2)
getVarCov(fit1,"conditional",individuals = 1)
round(AIC(fit1),digits = 1)
round(BIC(fit1),digits = 1)
round(logLik(fit1),digits = 1)
#======================================================================#


fit2 <- lme( Diameter ~ I(as.numeric(Group == 'AGA')) + 
               I(as.numeric(Group == 'SGA')) + 
               I(as.numeric(I(Group == 'AGA'))*month) +
               I(as.numeric(I(Group == 'SGA'))*month) +
               I(as.numeric(I(Group == 'SGA'))*(month ^ 2)) - 1,
             na.action = na.omit, 
             data = vecd,
             random = list(id = pdBlocked(list(
               pdSymm(~as.numeric(I(Group == 'AGA')) + 
                        I(as.numeric(I(Group == 'AGA'))*month) - 1),
               pdSymm(~as.numeric(I(Group == 'SGA')) + 
                        I(as.numeric(I(Group == 'SGA'))*month) + 
                        I(as.numeric(I(Group == 'SGA'))*(month ^ 2)) - 1))
             )
             ),
             
             control = list(MaxIter = 100000,niterEM = 100000,
                            msMaxIter = 100000,
                            tolerance = 0.0001,msTol = 0.0000001,
                            msMaxEval = 1000000)) 
summary(fit2)


