library(MASS)
library(Matrix)
library(mvtnorm)
library(fMultivar)
library(mnormt)
library(lattice)
#library(lme4)
library(nlme)
library(RLRsim)
library(StatDA)
library(car)

#====================================================================#
## Windows

#dados<-read.table("F:/2012/Artigo_Julio&Rocha/Programas simula??o/dados_PIG_AIG.txt",header=T,na="NA")

##

#dados<-read.table("//Users//franciscomarcelorocha//Documents//2012//Artigo_Julio&Rocha//Programas simulação//dados_PIG_AIG_f.txt",header=T,na="NA")
load("relspeed.RData")
dados <- relspeed
#====================================================================#
# Selecting data set
d  <-  split(dados,dados$Concentracao)
data <-  d$`16`
#====================================================================#
# Creating variables
x <-  seq(5,35,5)
xc <- x/60 # - mean(x)
xc2 <- xc ^ 2
xc3 <- xc ^ 3
xc4 <- xc ^ 4
nobs.data <- matrix(0,dim(data)[1],1)
np <- 3 # número de parâmetros
#====================================================================#

# Construindo o conjunto de dados para ajustar o polinomio do 3 grau

for (i in 1:dim(data)[1]) { 
   yi <-  (data[i,2:8])                       
  num.na <- length(yi[yi == 'NA'])
  num.val <- length(yi) - num.na
  nobs.data[i] <- num.val
}

data$nobs <-  nobs.data
# selecionando os dados para ajuste dos perfis individuais
data.fitind <- subset(data,data$nobs >= (np + 1))

#====================================================================#
#
######### Calculando as quantidades básicas para a utilização do método

m <-  dim(data.fitind)[1] # Número de indivíduos
peso <-  sqrt((m/(m - 1)))
nobst <- sum(data.fitind$nobs)
nu <- nobst - m*np
alfab <-  0.05/(m*np )

#====================================================================#
#######################################################################
# Polinômio 2 grau

tabela <- matrix(0,dim(data.fitind)[1],9)

for (i in 1:dim(data.fitind)[1]) {  
  
  yi <-  (data.fitind[i,2:8])
  nyi <- as.numeric(yi)                       
  fit <- lm(nyi~xc + xc2,na.action=na.omit)
  result <- summary(fit)
  sigma2i <- result$sigma ^ 2                        
  
  tabela[i,1:3] <-  fit$coef
  X <- model.matrix(fit)
  #   tabela[i,5:8] <-  diag(solve(t(X)%*%X))
  tabela[i,4:6] <-  (result$coeff[,2]/result$sigma ) ^ 2      #xtx-1
  tabela[i,7] <-  data.fitind[i,9]
  tabela[i,8] <-  sigma2i
  tabela[i,9] <-  sigma2i*(data.fitind[i,9] - np)
}
#====================================================================#
sigma2 <- sum(tabela[,9])/nu
#====================================================================#
# Fixed Effects
fixed.effects <- matrix(0,np,3)
#====================================================================#
# Intercept
fixed.effects[1,1] <-  mean(tabela[,1])
fixed.effects[1,2] <-  ((sqrt(sigma2*sum(tabela[,4])) )/m)
fixed.effects12 <-  fixed.effects[1,1]/fixed.effects[1,2]
fixed.effects[1,3] <- ifelse(fixed.effects12 > 0,
                             (2*pt(-fixed.effects12,df = nu)),
                             (2*pt(fixed.effects12,df = nu)))
#====================================================================#
# Slope
fixed.effects[2,1] <-  mean(tabela[,2])
fixed.effects[2,2] <-  ((sqrt(sigma2*sum(tabela[,5])) )/m)
fixed.effects22 <-  fixed.effects[2,1]/fixed.effects[2,2]
fixed.effects[2,3] <-  ifelse(fixed.effects22 > 0,
                              (2*pt(-fixed.effects22,df = nu)),
                              (2*pt(fixed.effects22,df = nu)))
#====================================================================#
 # Quadratic
fixed.effects[3,1] <-  mean(tabela[,3])
fixed.effects[3,2] <-  ((sqrt(sigma2*sum(tabela[,6])) )/m)
fixed.effects32 <-  fixed.effects[3,1]/fixed.effects[3,2]
fixed.effects[3,3] <-  ifelse(fixed.effects32 > 0,
                              (2*pt(-fixed.effects32,df = nu)),
                              (2*pt(fixed.effects32,df = nu)))
#====================================================================#
# Fixed effects estimates, standard error and p-value
fixedeffects <- data.frame(fixed.effects)
colnames(fixedeffects)  <-  c("Estimate", "s.e.","p-value")
fixedeffects
#====================================================================#
#
######################################################################
# Identifying random effects
######################################################################
# Matrix with w_k values
matrizw <- matrix(0,m,2*np)
alfab <-  0.05/(2*m*np )
# Incluindo na matriz as variáveis w
for (i in 1:np) {
  matrizw[,i] <-  tabela[,i]/sqrt(tabela[,(np + i)])  
}

for (j in 1:m) {
  for (k in 1:np) {
    matrizw[j,(k + np)] <-  (peso/sqrt(sigma2))*(matrizw[j,k] - 
                                                   mean(matrizw[,k]))
  }
}
#====================================================================#
aux <- matrix(0,m,np)
for (i in 4:6) { 
  for (j in 1:m) {
    aux[j,(i - np)] <- ifelse(matrizw[j,i] <= qt(alfab,nu) || 
                                matrizw[j,i] >= abs(qt(alfab,nu)),1,0 )
  }
}
# Identifying random effects
rand.effect <-  apply(aux,2,mean)
rand.effect



######################################################################
######################################################################
## Matriz de covariância
######################################################################
######################################################################

# Plot of the rows of the covariance matrix
dados1  <-  cov(data[,2:8])
plot(xc, dados1[1,], type = 'l', cex = 0.3, ylab = "Covariance",
     xlab = "Time (hours)")
for (i in 2:7) {
  lines(xc, dados1[i,], type = 'l', cex = 0.3)
}
#====================================================================#

data.cov <- dados1
tabela.cov <- matrix(0,dim(data.cov)[1],9)
#i <- 1
for (i in 1:dim(data.cov)[1]) {  
  
  yi <-  (data.cov[i,])
  nyi <- as.numeric(yi)                       
  fit <- lm(nyi~xc + xc2,na.action = na.omit)
  result <- summary(fit)
  sigma2i <- result$sigma ^ 2                        
  
  tabela.cov[i,1:3] <-  fit$coef
  X <- model.matrix(fit)
  #   tabela[i,5:8] <-  diag(solve(t(X)%*%X))
  tabela.cov[i,4:6] <-  (result$coeff[,2]/result$sigma ) ^ 2      #xtx-1
  tabela.cov[i,7] <-  data.fitind[i,9]
  tabela.cov[i,8] <-  sigma2i
  tabela.cov[i,9] <-  sigma2i*(data.fitind[i,9] - np)
}
#====================================================================#
sigma2.cov <- sum(tabela.cov[,9])/nu
#====================================================================#
# Creating variables
m.cov <-  dim(data.cov)[1]
nobst.cov <- (dim(data.cov)[1]) ^ 2
nu.cov <- nobst.cov - m.cov*np
alfab.cov <-  0.05/(2*m.cov*np )
peso.cov <-  sqrt((m.cov/(m.cov - 1)))
matrizw.cov <- matrix(0,m.cov,2*np)
#====================================================================#
# Matrix with w_k values
for (i in 1:np) {
  matrizw.cov[,i] <-  tabela.cov[,i]/sqrt(tabela.cov[,(np + i)])  
}

for (j in 1:m.cov) {
  for (k in 1:np) {
    matrizw.cov[j,(k + np)] <-  (peso.cov/sqrt(sigma2.cov))*
      (matrizw.cov[j,k] - mean(matrizw.cov[,k]))
  }
}

aux.cov <- matrix(0,m.cov,np)
for (i in 4:6) { 
  for (j in 1:m.cov) {
    aux.cov[j,(i - np)] <-  ifelse(matrizw.cov[j,i] <= 
                                     qt(alfab.cov,nu.cov) || 
                                     matrizw.cov[j,i] >= 
                                     abs(qt(alfab.cov,nu.cov)),1,0 )
  }
}
#====================================================================#
# Identifying random effects
rand.effect.cov <-  apply(aux.cov,2,mean)
rand.effect.cov


######################################################################
######################################################################
######################################################################
######################################################################
amostralcov <- abs(vech(dados1))
ajustadacov <- abs(vech(as.matrix(getVarCov(fm,"marginal",individuals=1)[[1]])))
max(abs(ajustadacov-amostralcov))
(mean(ajustadacov)-mean(amostralcov))