library(MASS)
library(Matrix)
library(mvtnorm)
library(fMultivar)
library(mnormt)
library(lattice)
library(lme4)
library(nlme)
library(RLRsim)
library(StatDA)
library(car)

#====================================================================#
#Leitura do conjunto de dados
## Windows


##
setwd("~/Documents/2016/Artigos/Artigo_Julio&Rocha/
      artigo sweave e rstudio/ArtigoJAS/Example1AGA")

#dados <- read.table("//Users//franciscomarcelorocha//Documents//2012
#//Artigo_Julio&Rocha//Programas simulação//dados_PIG_sga_f.txt",
#header=T,na="NA")

dados <- read.table("dados_PIG_AIG_f.txt",header = T,na = "NA")
#====================================================================#
# Separando os dados em AGA e SGA

d <- split(dados,dados$Group)
aig <- d$AGA
sga <- d$SGA

#====================================================================#
# Criando variáveis para ajuste dos perfis individuais

x <- 26:40
# Changing week variable to month
#xc <- (x - 26) / 4.29  
xc <- (x - mean(x)) / 4.29  
#xc <-  x  - mean(x)
xc2 <- xc ^ 2
xc3 <- xc ^ 3
np <- 3 # número de parâmetros

######################################################################
############################ Analisando AGA ##########################
######################################################################
nobs.sga <- matrix(0,dim(sga)[1],1)
# Construindo o conjunto de dados para ajustar o polin?mio do 3 grau

for (i in 1:dim(sga)[1]) { 
   yi <- (sga[i,3:17])                       
  num.na <- length(yi[yi == 'NA'])
  num.val <- length(yi) - num.na
  nobs.sga[i] <- num.val
}
sga$nobs <- nobs.sga

#====================================================================#

# selecionando os dados para ajuste dos perfis individuais
sga.fitind <- subset(sga,sga$nobs >= (np + 1))

# Gráfico
# 
min.aux <- c("week26","week27","week28","week29","week30","week31",
             "week32","week33","week34","week35","week36","week37",
             "week38","week39","week40")

vecsga <- reshape(sga, direction = "long", varying = 3:17,
                  v.names = "Diameter",timevar = "Weeks",
                  time = as.factor(min.aux))
vecsga <- vecsga[order(vecsga$id), ]
vecsga$weekv <- rep(xc, dim(sga)[1], each = T)
vecsga$subject <- vecsga$indf

vecsga <- na.exclude(vecsga)

gsga <- xyplot(Diameter ~ weekv |  Group, groups = id, pch = 16,
           par.settings = standard.theme(quartz,color = F),
            scales = list(x = list(relation = 'same'), 
                          y = list(relation = 'same')),type = 'l',
           lty = 1,as.table = TRUE,
            panel = function(x, y, col, ...) {
              panel.xyplot(x, y, col = col, ...)
              panel.average(x, y, fun = mean, horizontal = F,
                            lwd = 4, col = 'black', ...)   
            },
            ylab = "Aorta diameter (mm/kg)", 
           xlab = expression(bold(paste("Mean Profile in bold"))),
           na.action = na.omit,data = vecsga)



#====================================================================#
# excluded individuals treatment AGA
ind.exclude.sga <- dim(subset(sga,sga$nobs < (np + 1)))[1]
ind.exclude.sga

#====================================================================#
#
#### Calculando as quantidades básicas para a utilização do método ###

m <- dim(sga.fitind)[1] # Número de indivíduos
peso <- sqrt((m /(m - 1)))
nobst <- sum(sga.fitind$nobs)
#peso.cov<- sqrt((nobs/(nobs-1)))
nu <- nobst - m*np
alfab <- 0.05/(m*np )
#g.cov<-nobs^2 - nobs*np

#====================================================================#
#
## Ajustando modelo normal linear para sga ##

tabela.sga <- matrix(0,dim(sga.fitind)[1],(3*np))
mx <- matrix(c(xc,xc2,xc3),length(xc),(np - 1))
for (i in 1:dim(sga.fitind)[1]) {  
                         
  yi <- (sga.fitind[i,3:17])
  nyi <- as.numeric(yi)                       
  fit.sga.ind <- lm(nyi ~ mx[,1:(np - 1)] ,na.action = na.omit)
  result.sga <- summary(fit.sga.ind)
  sigma2i <- result.sga$sigma ^ 2                        

  tabela.sga[i,1:np] <- fit.sga.ind$coef
   X <- model.matrix(fit.sga.ind)
#   tabela[i,5:8]<- diag(solve(t(X)%*%X))
  tabela.sga[i,(np + 1):(2*np)] <- (result.sga$coeff[,2]/result.sga$sigma ) ^ 2 #xitxi
  tabela.sga[i,(2*np + 1)] <- sga.fitind[i,18]
  tabela.sga[i,(2*np + 2)] <- sigma2i
  tabela.sga[i,(2*np + 3)] <- sigma2i*(sga.fitind[i,18] - np)
}

#====================================================================#
# Estimate of the Conditional variance 
sigma2 <- sum(tabela.sga[,9])/nu
#====================================================================#
#
#  Efeitos Fixos

sga.fixed.effects <- matrix(0,np,3)

# Linear coefficient
sga.fixed.effects[1,1] <- mean(tabela.sga[,1])
sga.fixed.effects[1,2] <- ((sqrt(sigma2*sum(tabela.sga[,(np + 1)])) )/m)
sga.fixed.effects12 <- sga.fixed.effects[1,1] / sga.fixed.effects[1,2]
sga.fixed.effects[1,3] <- ifelse(sga.fixed.effects12 > 0,
                                 (2*pt(-sga.fixed.effects12,df = nu)),
                                 (2*pt(sga.fixed.effects12,df = nu)))
#====================================================================#
 # Slope coefficient
sga.fixed.effects[2,1] <- mean(tabela.sga[,2])
sga.fixed.effects[2,2] <- ((sqrt(sigma2*sum(tabela.sga[,(np + 2)])) )/m)
sga.fixed.effects22 <- sga.fixed.effects[2,1] / sga.fixed.effects[2,2]
sga.fixed.effects[2,3] <- ifelse(sga.fixed.effects22 > 0,
                             (2*pt(-sga.fixed.effects22,df = nu)),
                             (2*pt(sga.fixed.effects22,df = nu)))
#====================================================================#
# Quadratic coefficient
sga.fixed.effects[3,1] <- mean(tabela.sga[,3])
sga.fixed.effects[3,2] <- ((sqrt(sigma2*sum(tabela.sga[,(np + 3)])) )/m)
sga.fixed.effects32 <- sga.fixed.effects[3,1] / sga.fixed.effects[3,2]
sga.fixed.effects[3,3] <- ifelse(sga.fixed.effects32 > 0,
                                 (2*pt(-sga.fixed.effects32,df = nu)),
                                 (2*pt(sga.fixed.effects32,df = nu)))
#====================================================================#
# Cubic coefficient
sga.fixed.effects[4,1] <- mean(tabela.sga[,4])
sga.fixed.effects[4,2] <- ((sqrt(sigma2*sum(tabela.sga[,(np + 4)])) )/m)
sga.fixed.effects42 <- sga.fixed.effects[4,1] / sga.fixed.effects[4,2]
sga.fixed.effects[4,3] <- ifelse(sga.fixed.effects42 > 0,
                                 (2*pt(-sga.fixed.effects42,df = nu)),
                                 (2*pt(sga.fixed.effects42,df = nu)))

sga.fixedeffects <- data.frame(sga.fixed.effects)
colnames(sga.fixedeffects) <- c("Estimate", "s.e.","p-value")
sga.fixedeffects

######################################################################

# Random effects identifications

######################################################################

matrizw <- matrix(0,m,2*np)
# Incluindo na matriz as variáveis w
for (i in 1:np) {
  matrizw[,i] <- tabela.sga[,i]/sqrt(tabela.sga[,(np + i)])  
}

for (j in 1:m) {
  for (k in 1:np) {
    matrizw[j,(k + np)] <- (peso/sqrt(sigma2))*(matrizw[j,k] - 
                                                  mean(matrizw[,k]))
  }
}

aux <- matrix(0,m,np)
for (i in 4:6) { 
  for (j in 1:m) {
    aux[j,(i - np)] <- ifelse(matrizw[j,i] <= qt(alfab,nu) || 
                                matrizw[j,i] >= abs(qt(alfab,nu)),1,0)
  }
}

rand.effect <- apply(aux,2,mean)
rand.effect

######################################################################
######################################################################
######################################################################
